-------------------------------------------------------------------------------
-- |> Events
-- Manages all events
-------------------------------------------------------------------------------
-- |> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local UnitIsEnemy = UnitIsEnemy
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local GetTime = GetTime
local UnitIsPlayer = UnitIsPlayer
local UnitLevel = UnitLevel
local IsInInstance = IsInInstance
local bit_band = bit.band
local C_Timer_After = C_Timer.After
local InCombatLockdown = InCombatLockdown
local CombatLogGetCurrentEventInfo = CombatLogGetCurrentEventInfo
local UnitGUID = UnitGUID
local UnitHealth = UnitHealth
local UnitHealthMax = UnitHealthMax
local _
local UPDATE_INT = 0.5 -- > UNIT_HEALTH UPDATE THROTTLE

-------------------------------------------------------------------------------
-- |> FUNCTIONS
-------------------------------------------------------------------------------
-- > timestamp for updated unit_health to keep throtling in checmp3 install
local lastUpdated = {}
-----------------------------------------------------------------------
local function timestampUpdated(guid, newTime)
    if lastUpdated[guid] then
        lastUpdated[guid] = newTime
        return
    end
    lastUpdated[guid] = {}
    lastUpdated[guid] = newTime
end

-- > Static Role Assignment
---------------------------------------------------------------------
local role
function weizPVP:RoleAssign(class)
    role = nil
    if class == "ROGUE" or class == "MAGE" or class == "WARLOCK" or class == "HUNTER" then
        role = "DAMAGER"
    end
    if role then
        return role
    else
        return false
    end
end

-----------------------------------------------------------------------
-- > Get Spec from Spell
-----------------------------------------------------------------------
local specializationID
function weizPVP:GetSpecFromSpell(SpellID)
    specializationID = weizPVP.SpecDB[SpellID]
    return specializationID
end

-----------------------------------------------------------------------
-- > Get Spec from Aura
-----------------------------------------------------------------------
function weizPVP:GetSpecFromAura(SpellID)
    specializationID = weizPVP.AuraSpecDB[SpellID]
    return specializationID
end

-----------------------------------------------------------------------
-- > Get Role from Spec
-----------------------------------------------------------------------
function weizPVP:GetSpecRole(specid)
    if specid then
        _, _, _, _, role, _, _ = GetSpecializationInfoByID(specid)
        return role
    else
        return false
    end
end

-------------------------------------------------------------------------------------------
-- |> DETERMINE ZONE AND INSTANCE TYPE FROM EVENTS
-------------------------------------------------------------------------------------------
local inInstance
local WarModeEnabled
--> Check for Valid (PVP) Instance
-----------------------------------------------------------------------
function weizPVP:CheckValidInstance()
    -- > Checking for Valid BG
    inInstance = select(2, IsInInstance()) -- > Get instance info
    if inInstance == "pvp" and weizPVP.Options.EnabledInBattlegrounds == true then
        return true
    end

    if inInstance == "arena" == true and weizPVP.Options.EnabledInArena == true then
        return true
    end
    return false
end

--> Check for Valid World Zone
-----------------------------------------------------------------------
function weizPVP:CheckValidWorldZone()
    -- > Checking for Valid WPVP Zone
    inInstance = select(2, IsInInstance())
    WarModeEnabled = UnitIsWarModeDesired("player")
    if inInstance == "none" and WarModeEnabled == true then
        return true
    else
        return false
    end
end

--> Get PVP Zone
-----------------------------------------------------------------------
local ZoneTimerStartTime = GetTime()
local ValidBG
local ValidWorldZone
function weizPVP.GetPVPZone()
    if ZoneTimerStartTime + 1 <= GetTime() then
        ZoneTimerStartTime = GetTime()
        if GetZoneText() == "" then
            C_Timer_After(0.5, weizPVP.GetPVPZone)
            return
        end
        ValidWorldZone = weizPVP:CheckValidWorldZone()
        ValidBG = weizPVP.CheckValidInstance()
        if not InCombatLockdown() then
            if ValidWorldZone or ValidBG then
                weizPVP.OnEnable()
            else
                weizPVP.OnDisable()
            end
        end
    else
        return
    end
end

-------------------------------------------------------------------------------
-- |> ZONES
-------------------------------------------------------------------------------
--=> EVENT: Entering Battleground Instance
-----------------------------------------------------------------------
function weizPVP:PlayerEnteringBattlegroundEvent()
    C_Timer_After(1, weizPVP.GetPVPZone)
    wipe(weizPVP.CurrentNameplates)
end

-----------------------------------------------------------------------
--=> EVENT: ZoneChangedNewAreaEvent
-----------------------------------------------------------------------
function weizPVP:ZoneChangedNewAreaEvent()
    C_Timer_After(1, weizPVP.GetPVPZone)
end

-----------------------------------------------------------------------
--=> EVENT: Player Entering World
-----------------------------------------------------------------------
function weizPVP:PlayerEnteringWorldEvent()
    wipe(weizPVP.CurrentNameplates)
    C_Timer_After(1, weizPVP.GetPVPZone)
end

-----------------------------------------------------------------------
--=> EVENT TIMER; ONE SECOND REPEATING
-----------------------------------------------------------------------
function weizPVP:OneSecondTimer()
    -- > FPS Update
    weizPVP.Display.CurrentFPS = GetFramerate()

    -- > Check for LagMode
    weizPVP:LagModeCheck()

    -- > Manage Timeout Lists
    weizPVP.ManageListTimeouts()
end

--> Role Check
-----------------------------------------------------------------------
local dead
local GUID
local timestamp
local class
local newhealth
local _, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, arg12, arg13
-----------------------------------------------------------------------
local function ClRoleCheck(sGuid, spellID, spellName)
    role = nil

    -- > AURA
    if sGuid then
        specializationID = weizPVP:GetSpecFromAura(arg12) or nil
    end

    -- > Spell
    if not specializationID then
        specializationID = weizPVP:GetSpecFromSpell(arg12) or nil
    end

    if specializationID == nil then
        return "UNKNOWN"
    end

    role = weizPVP:GetSpecRole(specializationID) -- > "DAMAGER", "TANK", or "HEALER"
    if role then
        class = select(2, GetPlayerInfoByGUID(sGuid))
        return role
    end
    weizPVP:UpdatePlayerDB(srcName, class, nil, nil, nil, true, false, role)

    return "UNKNOWN"
end

--> STEALTH CHECK
-----------------------------------------------------------------------
local event
local stealthFound
local classColor = {1, 1, 1}
local stealthIDs = {1784, 115834, 114018, 11327, 5215, 66, 156136}
-----------------------------------------------------------------------
function weizPVP:CLStealthCheck(event, arg12, arg13, srcGUID, srcName)
    -- > Enabled?
    if weizPVP.Options.StealthAlert.Enabled ~= true and event == "SPELL_AURA_APPLIED" then
        return false
    end

    -- > Check for match
    stealthFound = false
    for i = 1, #stealthIDs do
        if stealthIDs[i] == arg12 then
            stealthFound = true
            break
        end
    end

    -- > RETURN IF NOT STEALTH
    if stealthFound == false then
        return false
    else
        class = select(2, GetPlayerInfoByGUID(srcGUID))
        classColor = RAID_CLASS_COLORS[class] or {r = 1, g = 1, b = 1}
        if not weizPVP.PlayerDB[srcName] then
            weizPVP:UpdatePlayerDB(arg13, nil, nil, nil, nil, true, false)
        end
        -- > Show Stealth Alert
        weizPVP:ShowStealthAlert(arg13 .. " Detected!", srcName, classColor.r, classColor.g, classColor.b)
        return true
    end
end

-----------------------------------------------------------------------
-- => EVENT TIMER; ONE SECOND REPEATING
-----------------------------------------------------------------------
local function ValidLogPlayer(flags, guid, name)
    if (bit_band(flags, 0x00000040) ~= 0 or weizPVP.Options.GLOBAL_DEBUG) and bit_band(flags, 0x00000400) ~= 0 and guid and name ~= "Unknown" then
        return true
    else
        return false
    end
end

-------------------------------------------------------------------------------------------
-- |> EVENTS: COMBAT LOG
-------------------------------------------------------------------------------------------
--=> CL: START
local stealth
-----------------------------------------------------------------------
function weizPVP:CombatLogEvent()
    _, event, _, srcGUID, srcName, srcFlags, _, dstGUID, dstName, dstFlags, _, arg12, arg13, _, _, _ = CombatLogGetCurrentEventInfo()

    -- > IS VALID SOURCE?
    -------------------------
    if ValidLogPlayer(srcFlags, srcGUID, srcName) then
        -- > VALID SOURCE; GET INFO
        -- > If Lag Mode is Off : Parse for unknown info
        if weizPVP.Options.LagModeActive == false then
            if not weizPVP.PlayerDB[srcName] or weizPVP.PlayerDB[srcName].Estimated == true then
                weizPVP:CLParseSource(event, srcName, srcGUID, arg12, arg13, srcFlags) -- > Parse data we don't have yet
            end
        end
        -- > Check stealth and role
        stealth = weizPVP:CLStealthCheck(event, arg12, arg13, srcGUID, srcName)
        role = ClRoleCheck(srcGUID, arg12, arg13)

        --> Update Source
        weizPVP:UpdatePlayerActiveCache(srcName, nil, nil, stealth, nil, role, srcGUID)
    -- UpdatePlayerActiveCache(name, newDataFound, health, stealth, dead, role, GUID)
    end

    -- > VALID DST; CHECK DEAD
    -------------------------
    if ValidLogPlayer(dstFlags, dstGUID, dstName) then
        weizPVP:CLDeath(event, dstGUID, dstName)
    end
end
-----------------------------------------------------------------------
-- > CL VALID SOURCE CHECK
-----------------------------------------------------------------------
local roleTime = 200
function weizPVP:CLParseSource(event, srcName, srcGUID, arg12, arg13, srcFlags)
    -- > Check if unit is not stored DB
    -----------------------------------------
    if not weizPVP.PlayerDB[srcName] and srcGUID then
        -- > Level check if Estimated
        -----------------------------------------
        role = nil
        if weizPVP.PlayerActiveCache[srcGUID] and weizPVP.PlayerActiveCache[srcGUID].Estimated == true then
            if weizPVP.PlayerDB[srcName].Timestamp + roleTime < GetTime() then
                -- > Role Check if estimated or its been a long time
                -----------------------------------------
                role = ClRoleCheck(srcGUID, arg12, arg13)
                weizPVP:GetLevelData(true, event, srcName, srcGUID, arg12, arg13)
            end
        end

        class = select(2, GetPlayerInfoByGUID(srcGUID))
        weizPVP:UpdatePlayerDB(srcName, class, nil, nil, nil, true, false, role)
    end

    -- > Check if unit is not in active cache
    -----------------------------------------
    if not weizPVP.PlayerActiveCache[srcGUID] then
        weizPVP:AddPlayerDataToNearby(srcGUID, false)
        return
    end
    weizPVP:UpdatePlayerDB(srcName, class, nil, nil, nil, true, false, role)
end

-------------------------------------------------------------------------------------------
-- |> EVENTS: UNITS
-------------------------------------------------------------------------------------------
-- => UNITS: MOUSEOVER
-----------------------------------------------------------------------
function weizPVP:PlayerMouseoverEvent()
    if not weizPVP.Options.LagModeActive and UnitExists("mouseover") then
        weizPVP:GetUnitData("mouseover")
    end
end

-- => UNITS: TARGET CHANGED
-- -----------------------------------------------------------------------
function weizPVP:PlayerTargetEvent()
    weizPVP:GetUnitData("target")
    weizPVP:ChangeTargetIcon()
end

-- => UNITS: NAMEPLATE ADDED
-- -----------------------------------------------------------------------
function weizPVP:NameplateAdded(_, unit)
    if (UnitIsEnemy("player", unit) or weizPVP.Options.GLOBAL_DEBUG) and UnitIsPlayer(unit) then
        GUID = UnitGUID(unit)
        -- > Update time only
        if weizPVP.PlayerActiveCache[GUID] and weizPVP.PlayerActiveCache[GUID].Estimated == false then
            weizPVP.PlayerActiveCache[GUID].Stealth = false
            weizPVP:AddPlayerDataToNearby(GUID, false)
        else
            weizPVP:GetUnitData(unit)
        end
        weizPVP.CurrentNameplates[GUID] = true
    end
end

-----------------------------------------------------------------------
-- => UNITS: NAMEPLATE REMOVED
-----------------------------------------------------------------------
function weizPVP:NameplateRemoved(_, unit)
    GUID = UnitGUID(unit)
    if weizPVP.PlayerActiveCache[GUID] then
        weizPVP:AddPlayerDataToNearby(GUID, false)
    end
    weizPVP.CurrentNameplates[GUID] = nil
end

-----------------------------------------------------------------------
-- => EVENT: UNIT_HEALTH
-----------------------------------------------------------------------
function weizPVP:UnitHealthEvent(_, unit)
    if unit and UnitGUID(unit) then
        GUID = UnitGUID(unit)
        -- > Check if player is aleady on bars
        -------------------------------------
        if weizPVP.PlayersOnBars[GUID] then
            -- > Get unit info
            -------------------------------------
            timestamp = GetTime()
            newhealth = UnitHealth(unit) / UnitHealthMax(unit) * 100
            dead = UnitIsDeadOrGhost(unit)

            -- > Check for dead player
            -------------------------------------
            if weizPVP.PlayerActiveCache[GUID] and dead ~= weizPVP.PlayerActiveCache[GUID].Dead then
                weizPVP.PlayerActiveCache[GUID].Dead = dead
                weizPVP:AddPlayerDataToNearby(GUID, false)
                timestampUpdated(GUID, timestamp)
                return
            elseif dead == false and weizPVP.PlayerActiveCache[GUID].Dead == true then
                weizPVP.PlayerActiveCache[GUID].Dead = dead
                if newhealth == 0 then
                    weizPVP.PlayerActiveCache[GUID].Health = 10
                end
                weizPVP:AddPlayerDataToNearby(GUID, false)
                timestampUpdated(GUID, timestamp)
                return
            end

            -- > Check if player is aleady on bars
            -------------------------------------
            if newhealth and weizPVP.PlayerActiveCache[GUID].Health ~= newhealth then
                weizPVP.PlayerActiveCache[GUID].Health = newhealth
            end

            -- > Update Health is enough time has passed to update the bars
            -------------------------------------
            if not lastUpdated[GUID] or (lastUpdated[GUID] and lastUpdated[GUID] + UPDATE_INT < timestamp) then
                weizPVP:AddPlayerDataToNearby(GUID, false)
                timestampUpdated(GUID, timestamp)
            end
        else
            -- > Player was not active on bars, but we have their last time listed.
            -- > Clear this player's time from the table
            if lastUpdated[GUID] then
                lastUpdated[GUID] = nil
            end
        end
    end
end

-------------------------------------------------------------------------------------------
-- |> EVENTS: COMBAT
-------------------------------------------------------------------------------------------
-- => EVENT: ENTERING COMBAT
-----------------------------------------------------------------------
function weizPVP:EnteringCombat()
    weizPVP:SetStatusBarMessage("|cFFF4564D +++ IN COMBAT +++|r", false, 0)
end

-----------------------------------------------------------------------
-- => EVENT: LEAVING COMBAT
-----------------------------------------------------------------------
function weizPVP:LeavingCombat()
    weizPVP:SetStatusBarMessage("|cFF53F493 --- OUT OF COMBAT ---|r", true, 3)
    weizPVP:ManageListTimeouts()
end

-----------------------------------------------------------------------
-- => EVENT: PLAYER LEVELED UP
-----------------------------------------------------------------------
function weizPVP:PlayerLevelUp()
    weizPVP.Player.Level = UnitLevel("player") -- > Update player level
end

-- > COMBAT LOG DEATH CHECK
-----------------------------------------------------------------------
function weizPVP:CLDeath(event, dstGUID, dstName)
    if weizPVP.PlayerActiveCache[dstGUID] then
        if event == "UNIT_DIED" then
            weizPVP.PlayerActiveCache[dstGUID].Dead = true
            weizPVP:UpdatePlayerActiveCache(dstName, nil, nil, nil, true, nil, dstGUID)
            return true
        end
    end
    return false -- > return if both src and dst are known
end

-- > ENABLE EVENTS
-----------------------------------------------------------------------
function weizPVP.EnableEvents()
    -- > DATA COLLECTION
    weizPVP:RegisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
    weizPVP:RegisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
    weizPVP:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
    weizPVP:RegisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
    weizPVP:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
    weizPVP:RegisterEvent("UNIT_HEALTH", "UnitHealthEvent")

    -- > ZONE CHANGED
    weizPVP:RegisterEvent("PLAYER_ENTERING_BATTLEGROUND", "PlayerEnteringBattlegroundEvent")
    weizPVP:RegisterEvent("PLAYER_ENTERING_WORLD", "PlayerEnteringWorldEvent")
    weizPVP:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneChangedNewAreaEvent")

    -- > COMBAT/PVP
    weizPVP:RegisterEvent("PLAYER_REGEN_DISABLED", "EnteringCombat")
    weizPVP:RegisterEvent("PLAYER_REGEN_ENABLED", "LeavingCombat")

    -- > PLAYER UPDATES
    weizPVP:RegisterEvent("PLAYER_LEVEL_UP", "PlayerLevelUp")

    -- > DISPLAY CHANGES
    weizPVP:RegisterEvent("DISPLAY_SIZE_CHANGED", "UpdateDisplayData")
    weizPVP:RegisterEvent("PLAYER_LOGIN", "UpdateDisplayData")
end

-- > DISABLE EVENTS
-----------------------------------------------------------------------
function weizPVP.DisableEvents()
    -- > DATA COLLECTION
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
    weizPVP:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
    weizPVP:UnregisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
    weizPVP:UnregisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
    weizPVP:UnregisterEvent("UNIT_HEALTH", "UnitHealthEvent")

    -- > ZONE CHANGED
    -- Zone events do not get disabled. Needed to auto-enable.

    -- > COMBAT/PVP
    weizPVP:UnregisterEvent("PLAYER_REGEN_DISABLED", "EnteringCombat")
    weizPVP:UnregisterEvent("PLAYER_REGEN_ENABLED", "LeavingCombat")

    -- > PLAYER UPDATES
    weizPVP:UnregisterEvent("PLAYER_LEVEL_UP", "PlayerLevelUp")

    -- > DISPLAY CHANGES
    weizPVP:UnregisterEvent("DISPLAY_SIZE_CHANGED", "UpdateDisplayData")
    weizPVP:UnregisterEvent("PLAYER_LOGIN", "UpdateDisplayData")
end

-------------------------------------------------------------------------------------------
-- |>   LAGMODE CHECK
-------------------------------------------------------------------------------------------
local LagModeCounter = 0

function weizPVP:LagModeCheck()
    if weizPVP.Options.LagMode.Enabled == true then
        if LagModeCounter >= 4 then
            LagModeCounter = 0
            if weizPVP.Display.CurrentFPS < weizPVP.Options.LagMode.FPSThreshold and weizPVP.NearbyListSize > 12 then
                if weizPVP.Options.LagModeActive == false then
                    -- > LAGMODE ON
                    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
                    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
                    weizPVP.CurrentNameplates = {}
                    weizPVP.Options.LagModeActive = true
                    UPDATE_INT = 1
                end
            elseif weizPVP.Display.CurrentFPS > weizPVP.Options.LagMode.FPSThreshold + 4 then
                if weizPVP.Options.LagModeActive == true then
                    -- > LAGMODE OFF
                    weizPVP:RegisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
                    weizPVP:RegisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
                    weizPVP.Options.LagModeActive = false
                    UPDATE_INT = 0.5
                end
            end
        else
            LagModeCounter = LagModeCounter + 1
        end
    end
end
